/*
 * baudrate.c:
 * Convert between integer baud rates and the TERMIOS constants.
 *
 * Copyright (c) 2004 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 */

static const char rcsid[] = "$Id: baudrate.c,v 1.2 2004/06/22 19:22:46 chris Exp $";

#include <termios.h>
#include <unistd.h>

/* n_to_rate N
 * Return the proper B... termios constant corresponding to the integer
 * speed N; or B0 on failure. */
speed_t n_to_rate(const int n) {
    switch (n) {
#define RATE(n)     case n: return B##n
        RATE(50);
        RATE(75);
        RATE(110);
        RATE(134);
        RATE(150);
        RATE(200);
        RATE(300);
        RATE(600);
        RATE(1200);
        RATE(1800);
        RATE(2400);
        RATE(4800);
        RATE(9600);
        RATE(19200);
        RATE(38400);
        RATE(57600);
        RATE(115200);
        RATE(230400);
#undef RATE
        default: return -1;
    }
}

int rate_to_n(const speed_t r) {
    switch (r) {
#define RATE(n)     case B##n: return n
        RATE(50);
        RATE(75);
        RATE(110);
        RATE(134);
        RATE(150);
        RATE(200);
        RATE(300);
        RATE(600);
        RATE(1200);
        RATE(1800);
        RATE(2400);
        RATE(4800);
        RATE(9600);
        RATE(19200);
        RATE(38400);
        RATE(57600);
        RATE(115200);
        RATE(230400);
#undef RATE
        default: return -1;

    }
}
