/*
 * consolidate.h:
 * Declarations for consolidate.
 *
 * Copyright (c) 2004 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 * $Id: consolidate.h,v 1.7 2004/08/15 17:06:28 chris Exp $
 *
 */

#ifndef __CONSOLIDATE_H_ /* include guard */
#define __CONSOLIDATE_H_

#include <sys/types.h>

#include <stdio.h>
#include <syslog.h>
#include <termios.h>
#include <unistd.h>

/* baudrate.c */
speed_t n_to_rate(const int n);
int rate_to_n(const speed_t r);

/* log.c */
void log_print(const int pri, const char *fmt, ...);

/* io.c */
ssize_t xwrite(int fd, const char *buf, size_t len);
ssize_t xread(int fd, char *buf, size_t len);
int write_to_tty(int fd, const char *buf, const size_t len);
int write_to_client(const char *tty, int fd, const char *buf, const size_t len);
int write_to_logfile(const char *tty, const char *name, int fd, const char *line, size_t len);

/* logrotate.c */
int rotate_logfile(const char *logfile, const int gzip, const unsigned int max);

/* uucplock.c */
int uucp_lock(const char *dev);
void uucp_unlock(const char *dev);

/* util.h */
void *xmalloc(size_t n);
void *xcalloc(size_t n, size_t m);
void *xrealloc(void *w, size_t n);
char *xstrdup(const char *s);
void xfree(void *v);
void (*xsignal(int signum, void(*handler)(int)))(int);
char *sprintf_new(const char *fmt, ...);
char *get_line(FILE *fp);

/* alloc_struct TAG P
 * Enough memory to hold a struct TAG is allocated and assigned to P, and the
 * memory is initialised so that all numeric elements of *P are zero and all
 * pointer elements NULL. */
#define alloc_struct(tag, p)                    \
            do {                                \
                struct tag aszero ## tag = {0}; \
                p = malloc(sizeof *p);          \
                *p = aszero ## tag;             \
            } while (0)

#endif /* __CONSOLIDATE_H_ */
