/*
 * log.c:
 * Logging via system log and, optionally, standard error.
 *
 * Copyright (c) 2004 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 */

static const char rcsid[] = "$Id: log.c,v 1.3 2004/04/16 18:29:08 chris Exp $";

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>

#include "consolidate.h"

/* subst STRING OLD NEW
 * Return a copy of STRING in which each (non-overlapping) occurence of the
 * string OLD is replaced by NEW. */
static char *subst(const char *str, const char *old, const char *new) {
    static char *buf;
    static size_t buflen;
    size_t l, oldlen, newlen;
    const char *p;
    char *q;

    if (buflen < (l = 1 + (strlen(str) * (newlen = strlen(new))) / (oldlen = strlen(old))))
        buf = xrealloc(buf, buflen = l);
    
    for (p = str, q = buf; *p; ) {
        if (0 == strcmp(p, old)) {  /* XXX inefficient */
            strcpy(q, new);
            p += oldlen;
            q += newlen;
        } else
            *q++ = *p++;
    }

    *q = 0;

    return buf;
}

/* log_print PRIORITY FORMAT ...
 * Like syslog(3), but optionally also output to standard error. */
void log_print(const int pri, const char *fmt, ...) {
    static int opened;
    char *newfmt;
    extern int log_stderr;
    va_list ap;
    static char *buf;
    static size_t buflen;
    size_t l;

    if (!opened)
        openlog("consolidate", LOG_NDELAY | LOG_PID, LOG_DAEMON);

    newfmt = subst(fmt, "%m", strerror(errno));

    if (buflen < (l = strlen(newfmt) + 1))
        buf = xrealloc(buf, buflen = l);

    while (1) {
        int n;
        va_start(ap, fmt);
        n = vsnprintf(buf, buflen, newfmt, ap);
        va_end(ap);
        if (n > -1 && n < buflen)
            break;
        else if (n > -1)
            l = n + 1;
        else
            l = buflen * 2;
        buf = xrealloc(buf, buflen = l);
    }

    if (log_stderr)
        fprintf(stderr, "consolidate: %s\n", buf);
    
    syslog(pri | LOG_DAEMON, "%s", buf);
}

